@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -15,6 +15,7 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                           }....................
 from beartype._data.hint.pep.sign.datapepsigns import (
+    # PEP-agnostic signs.
     HintSignAbstractSet,
     HintSignAnnotated,
     HintSignAny,
@@ -68,6 +69,9 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignTypeVar,
     HintSignUnion,
     HintSignValuesView,
+
+    # PEP-specific signs.
+    HintSignDataclassInitVar,
 )
 
 # See the "beartype.cave" submodule for further commentary.
@@ -374,6 +378,9 @@ HINT_SIGNS_SUPPORTED_DEEP = frozenset((
     # ..................{ PEP 544                           }..................
     HintSignProtocol,
 
+    # ..................{ PEP 557                           }..................
+    HintSignDataclassInitVar,
+
     # ..................{ PEP 586                           }..................
     HintSignLiteral,
 
