@@ -20,14 +20,27 @@ This private submodule is *not* intended for importation by downstream callers.
 # All "FIXME:" comments for this submodule reside in this package's "__init__"
 # submodule to improve maintainability and readability here.
 
-#FIXME: Consider unifying this submodule with the "pepcode" submodule. Now
-#that *EVERYTHING* is effectively PEP-compliant, there's really *NO*
-#demonstrable reason to pretend the two are distinct workflows anymore.
+#FIXME: Split this large submodule into smaller submodules for maintainability.
+#A useful approach might be:
+#* Define a new private "_codearg" submodule and shift the _code_check_args()
+#  function there.
+#* Define a new private "_codereturn" submodule and shift the
+#  _code_check_return() function there.
 
 # ....................{ IMPORTS                           }....................
-from beartype.roar import BeartypeDecorParamNameException
+from beartype.roar import (
+    BeartypeDecorParamNameException,
+    BeartypeDecorHintPepException,
+)
+from beartype.typing import NoReturn
+from beartype._decor._cache.cachetype import (
+    bear_typistry,
+    register_typistry_forwardref,
+)
+from beartype._decor._call import BeartypeCall
 from beartype._decor._code.codemagic import (
     ARG_NAME_GETRANDBITS,
+    ARG_NAME_TYPISTRY,
     EXCEPTION_PREFIX,
 )
 from beartype._decor._code.codesnip import (
@@ -37,40 +50,53 @@ from beartype._decor._code.codesnip import (
     CODE_SIGNATURE,
     PEP484_CODE_CHECK_NORETURN,
 )
-from beartype._decor._code._pep.pepcode import (
-    pep_code_check_arg,
-    pep_code_check_return,
+from beartype._decor._code._pep._pephint import pep_code_check_hint
+from beartype._decor._code._pep._pepsnip import (
+    PARAM_KIND_TO_PEP_CODE_LOCALIZE,
+    PEP_CODE_CHECK_RETURN_PREFIX,
+    PEP_CODE_CHECK_RETURN_SUFFIX,
+    PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_PREFIX,
+    PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_SUFFIX,
+    PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
 )
-from beartype._decor._call import BeartypeCall
 from beartype._util.error.utilerror import reraise_exception_placeholder
 from beartype._util.func.arg.utilfuncargiter import (
-    ParameterKind,
+    # ARG_META_INDEX_DEFAULT,
+    ARG_META_INDEX_KIND,
+    ARG_META_INDEX_NAME,
+    ArgKind,
     iter_func_args,
 )
 from beartype._util.hint.pep.proposal.pep484585.utilpep484585func import (
     reduce_hint_pep484585_func_return)
+from beartype._util.hint.pep.proposal.pep484585.utilpep484585ref import (
+    get_hint_pep484585_forwardref_classname_relative_to_object)
 from beartype._util.hint.utilhintconv import sanify_hint_root
 from beartype._util.hint.utilhinttest import is_hint_ignorable
+from beartype._util.kind.utilkinddict import update_mapping
 from beartype._util.text.utiltextlabel import (
     prefix_callable_decorated_arg,
     prefix_callable_decorated_return,
 )
 from beartype._util.text.utiltextmagic import CODE_INDENT_1
+from beartype._util.text.utiltextmunge import replace_str_substrs
 from beartype._util.text.utiltextrepr import represent_object
 from beartype._util.utilobject import SENTINEL
-from collections.abc import Callable
-from typing import NoReturn
+from collections.abc import (
+    Callable,
+    Iterable,
+)
 
 # See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
-# ....................{ CONSTANTS ~ private               }....................
+# ....................{ PRIVATE ~ constants               }....................
 #FIXME: Remove this set *AFTER* handling these kinds of parameters.
 _PARAM_KINDS_IGNORABLE = frozenset((
-    ParameterKind.VAR_KEYWORD,
+    ArgKind.VAR_KEYWORD,
 ))
 '''
-Frozen set of all :attr:`ParameterKind` enumeration members to be ignored
+Frozen set of all :attr:`ArgKind` enumeration members to be ignored
 during annotation-based type checking in the :func:`beartype.beartype`
 decorator.
 
@@ -86,15 +112,23 @@ This includes:
 
 
 _PARAM_KINDS_POSITIONAL = frozenset((
-    ParameterKind.POSITIONAL_ONLY,
-    ParameterKind.POSITIONAL_OR_KEYWORD,
+    ArgKind.POSITIONAL_ONLY,
+    ArgKind.POSITIONAL_OR_KEYWORD,
 ))
 '''
 Frozen set of all **positional parameter kinds** (i.e.,
-:attr:`ParameterKind` enumeration members signifying that a callable parameter
+:attr:`ArgKind` enumeration members signifying that a callable parameter
 either may *or* must be passed positionally).
 '''
 
+
+_RETURN_REPR = repr('return')
+'''
+Object representation of the magic string implying a return value in various
+Python objects (e.g., the ``__annotations__`` dunder dictionary of annotated
+callables).
+'''
+
 # ....................{ GENERATORS                        }....................
 def generate_code(
     bear_call: BeartypeCall,
@@ -284,9 +318,6 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
     # Python code snippet to be returned.
     func_wrapper_code = ''
 
-    # Python code snippet type-checking the current parameter.
-    func_wrapper_code_arg = ''
-
     # ..................{ LOCALS ~ parameter                }..................
     #FIXME: Remove this *AFTER* optimizing signature generation, please.
     # True only if this callable possibly accepts one or more positional
@@ -303,43 +334,47 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
     #calling the enumerate() builtin here) AFTER* refactoring @beartype to
     #generate callable-specific wrapper signatures.
 
-    # Attempt to...
-    try:
-        # For the name of each parameter accepted by this callable and the
-        # "ParameterMeta" object describing this parameter (in declaration
-        # order)...
-        for arg_meta in iter_func_args(
-            # Possibly lowest-level wrappee underlying the possibly
-            # higher-level wrapper currently being decorated by the @beartype
-            # decorator. The latter typically fails to convey the same callable
-            # metadata conveyed by the former -- including the names and kinds
-            # of parameters accepted by the possibly unwrapped callable. This
-            # renders the latter mostly useless for our purposes.
-            func=bear_call.func_wrappee_wrappee,
-            func_codeobj=bear_call.func_wrappee_wrappee_codeobj,
-            is_unwrapping=False,
-        ):
-            # Type hint annotating this parameter if any *OR* the sentinel
-            # placeholder otherwise (i.e., if this parameter is unannotated).
-            #
-            # Note that "None" is a semantically meaningful PEP 484-compliant
-            # type hint equivalent to "type(None)". Ergo, we *MUST* explicitly
-            # distinguish between that type hint and unannotated parameters.
-            hint = bear_call.func_arg_name_to_hint_get(arg_meta.name, SENTINEL)
+    # For the name of each parameter accepted by this callable and the
+    # "ParameterMeta" object describing this parameter (in declaration
+    # order)...
+    for arg_index, arg_meta in enumerate(iter_func_args(
+        # Possibly lowest-level wrappee underlying the possibly
+        # higher-level wrapper currently being decorated by the @beartype
+        # decorator. The latter typically fails to convey the same callable
+        # metadata conveyed by the former -- including the names and kinds
+        # of parameters accepted by the possibly unwrapped callable. This
+        # renders the latter mostly useless for our purposes.
+        func=bear_call.func_wrappee_wrappee,
+        func_codeobj=bear_call.func_wrappee_wrappee_codeobj,
+        is_unwrapping=False,
+    )):
+        # Kind and name of this parameter.
+        arg_kind: ArgKind = arg_meta[ARG_META_INDEX_KIND]  # type: ignore[assignment]
+        arg_name: str = arg_meta[ARG_META_INDEX_NAME]  # type: ignore[assignment]
 
-            # If this parameter is unannotated, continue to the next parameter.
-            if hint is SENTINEL:
-                continue
-            # Else, this parameter is annotated.
+        # Type hint annotating this parameter if any *OR* the sentinel
+        # placeholder otherwise (i.e., if this parameter is unannotated).
+        #
+        # Note that "None" is a semantically meaningful PEP 484-compliant type
+        # hint equivalent to "type(None)". Ergo, we *MUST* explicitly
+        # distinguish between that type hint and unannotated parameters.
+        hint = bear_call.func_arg_name_to_hint_get(arg_name, SENTINEL)
 
+        # If this parameter is unannotated, continue to the next parameter.
+        if hint is SENTINEL:
+            continue
+        # Else, this parameter is annotated.
+
+        # Attempt to...
+        try:
             # If this parameter's name is reserved for use by the @beartype
             # decorator, raise an exception.
-            if arg_meta.name.startswith('__bear'):
+            if arg_name.startswith('__bear'):
                 raise BeartypeDecorParamNameException(
                     f'{EXCEPTION_PREFIX}reserved by @beartype.')
             # If either the type of this parameter is silently ignorable, continue
             # to the next parameter.
-            elif arg_meta.kind in _PARAM_KINDS_IGNORABLE:
+            elif arg_kind in _PARAM_KINDS_IGNORABLE:
                 continue
             # Else, this parameter is non-ignorable.
 
@@ -353,7 +388,7 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
             hint = sanify_hint_root(
                 hint=hint,
                 func=bear_call.func_wrappee,
-                pith_name=arg_meta.name,
+                pith_name=arg_name,
                 exception_prefix=EXCEPTION_PREFIX,
             )
 
@@ -374,28 +409,73 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
             # parameter to be unignorable, @beartype would fail to reduce to a
             # noop for otherwise ignorable callables -- which would be rather
             # bad, really.
-            elif arg_meta.kind in _PARAM_KINDS_POSITIONAL:
+            elif arg_kind in _PARAM_KINDS_POSITIONAL:
                 is_args_positional = True
 
-            # Python code snippet type-checking this parameter against this hint.
-            func_wrapper_code_arg = pep_code_check_arg(
-                bear_call=bear_call, hint=hint, arg_meta=arg_meta)
+            # Python code template localizing this parameter.
+            #
+            # Since @beartype now supports *ALL* parameter kinds, we safely
+            # assume this behaves as expected without additional validation.
+            # PARAM_LOCALIZE_TEMPLATE = PARAM_KIND_TO_PEP_CODE_LOCALIZE[arg_kind]
+
+            #FIXME: Preserved in the event of a new future unsupported parameter kind.
+            # Python code template localizing this parameter if this kind of
+            # parameter is supported *OR* "None" otherwise.
+            PARAM_LOCALIZE_TEMPLATE = PARAM_KIND_TO_PEP_CODE_LOCALIZE.get(  # type: ignore
+                arg_kind, None)
+
+            # If this kind of parameter is unsupported, raise an exception.
+            #
+            # Note this edge case should *NEVER* occur, as the parent function
+            # should have simply ignored this parameter.
+            if PARAM_LOCALIZE_TEMPLATE is None:
+                raise BeartypeDecorHintPepException(
+                    f'{EXCEPTION_PREFIX}kind {repr(arg_kind)} '
+                    f'currently unsupported by @beartype.'
+                )
+            # Else, this kind of parameter is supported. Ergo, this code is
+            # non-"None".
+
+            # Generate a memoized parameter-agnostic code snippet type-checking
+            # any parameter or return value with an arbitrary name.
+            (
+                code_param_check_pith,
+                func_wrapper_locals,
+                hint_forwardrefs_class_basename,
+            ) = pep_code_check_hint(hint)
+
+            # Merge the local scope required to type-check this parameter into the
+            # local scope currently required by the current wrapper function.
+            update_mapping(bear_call.func_wrapper_locals, func_wrapper_locals)
+
+            # Python code snippet localizing this parameter.
+            code_param_localize = PARAM_LOCALIZE_TEMPLATE.format(
+                arg_name=arg_name, arg_index=arg_index)
+
+            # Unmemoize this snippet against the current parameter.
+            code_param_check = _unmemoize_pep_code(
+                bear_call=bear_call,
+                func_wrapper_code=code_param_check_pith,
+                pith_repr=repr(arg_name),
+                hint_forwardrefs_class_basename=hint_forwardrefs_class_basename,
+            )
 
             # Append code type-checking this parameter against this hint.
-            func_wrapper_code += func_wrapper_code_arg
-    # If any exception was raised, reraise this exception with each placeholder
-    # (i.e., "beartype._util.error.utilerror.EXCEPTION_PLACEHOLDER"
-    # instance) replaced with a description of this callable and parameter.
-    except Exception as exception:
-        reraise_exception_placeholder(
-            exception=exception,
-            #FIXME: Embed the kind of parameter as well (e.g.,
-            #"positional-only", "keyword-only", "variadic positional"), ideally
-            #by improving the existing prefix_callable_decorated_arg()
-            #function to introspect this kind from that callable's code object.
-            target_str=prefix_callable_decorated_arg(
-                func=bear_call.func_wrappee, arg_name=arg_meta.name),
-        )
+            func_wrapper_code += f'{code_param_localize}{code_param_check}'
+        # If any exception was raised, reraise this exception with each
+        # placeholder substring (i.e.,
+        # "beartype._util.error.utilerror.EXCEPTION_PLACEHOLDER" instance)
+        # replaced with a description of this callable and parameter.
+        except Exception as exception:
+            reraise_exception_placeholder(
+                exception=exception,
+                #FIXME: Embed the kind of parameter as well (e.g.,
+                #"positional-only", "keyword-only", "variadic positional"), ideally
+                #by improving the existing prefix_callable_decorated_arg()
+                #function to introspect this kind from that callable's code object.
+                target_str=prefix_callable_decorated_arg(
+                    func=bear_call.func_wrappee, arg_name=arg_name),
+            )
 
     # If this callable accepts one or more positional type-checked parameters,
     # prefix this code by a snippet localizing the number of these parameters.
@@ -500,12 +580,50 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
             # If this PEP-compliant hint is unignorable, generate and return a
             # snippet type-checking this return against this hint.
             if not is_hint_ignorable(hint):
-                func_wrapper_code = pep_code_check_return(
-                    bear_call=bear_call, hint=hint)
+                # Empty tuple, passed below to satisfy the
+                # _unmemoize_pep_code() API.
+                hint_forwardrefs_class_basename = ()
+
+                # Generate a memoized parameter-agnostic code snippet
+                # type-checking any parameter or return with any name.
+                (
+                    code_return_check_pith,
+                    func_wrapper_locals,
+                    hint_forwardrefs_class_basename,
+                ) = pep_code_check_hint(hint)
+
+                # Merge the local scope required to type-check this return into
+                # the local scope currently required by the current wrapper
+                # function.
+                update_mapping(
+                    bear_call.func_wrapper_locals, func_wrapper_locals)
+
+                # Unmemoize this snippet against this return.
+                code_return_check_pith_unmemoized = _unmemoize_pep_code(
+                    bear_call=bear_call,
+                    func_wrapper_code=code_return_check_pith,
+                    pith_repr=_RETURN_REPR,
+                    hint_forwardrefs_class_basename=(
+                        hint_forwardrefs_class_basename),
+                )
+
+                # Python code snippet type-checking this return.
+                code_return_check_prefix = PEP_CODE_CHECK_RETURN_PREFIX.format(
+                    func_call_prefix=bear_call.func_wrapper_code_call_prefix)
+
+                # Return a Python code snippet:
+                # * Calling the decorated callable and localize its return *AND*...
+                # * Type-checking this return *AND*...
+                # * Returning this return from this wrapper function.
+                func_wrapper_code = (
+                    f'{code_return_check_prefix}'
+                    f'{code_return_check_pith_unmemoized}'
+                    f'{PEP_CODE_CHECK_RETURN_SUFFIX}'
+                )
             # Else, this PEP-compliant hint is ignorable.
             # if not func_wrapper_code: print(f'Ignoring {bear_call.func_name} return hint {repr(hint)}...')
     # If any exception was raised, reraise this exception with each placeholder
-    # (i.e., "beartype._util.error.utilerror.EXCEPTION_PLACEHOLDER"
+    # substring (i.e., "beartype._util.error.utilerror.EXCEPTION_PLACEHOLDER"
     # instance) replaced with a description of this callable and return.
     except Exception as exception:
         reraise_exception_placeholder(
@@ -603,3 +721,102 @@ def _make_func_wrapper_signature(
         f'{code_signature}'
         f'{code_body_init}'
     )
+
+# ....................{ PRIVATE ~ unmemoize               }....................
+def _unmemoize_pep_code(
+    bear_call: BeartypeCall,
+    func_wrapper_code: str,
+    pith_repr: str,
+    hint_forwardrefs_class_basename: tuple,
+) -> str:
+    '''
+    Convert the passed memoized code snippet type-checking any parameter or
+    return of the decorated callable into a memoized code snippet type-checking
+    a specific parameter or return of that callable.
+
+    Specifically, this function (in order):
+
+    #. Globally replaces all references to the
+       :data:`PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER` placeholder substring
+       cached into this code with the passed ``pith_repr`` parameter.
+    #. Unmemoizes this code by globally replacing all relative forward
+       reference placeholder substrings cached into this code with Python
+       expressions evaluating to the classes referred to by those substrings
+       relative to that callable when accessed via the private
+       ``__beartypistry`` parameter.
+
+    Parameters
+    ----------
+    bear_call : BeartypeCall
+        Decorated callable to be type-checked.
+    func_wrapper_code : str
+        Memoized callable-agnostic code snippet type-checking any parameter or
+        return of the decorated callable.
+    pith_repr : str
+        Machine-readable representation of the name of this parameter or
+        return.
+    hint_forwardrefs_class_basename : tuple
+        Tuple of the unqualified classnames referred to by all relative forward
+        reference type hints visitable from the current root type hint.
+
+    Returns
+    ----------
+    str
+        This memoized code unmemoized by globally resolving all relative
+        forward reference placeholder substrings cached into this code relative
+        to the currently decorated callable.
+    '''
+    assert bear_call.__class__ is BeartypeCall, (
+        f'{repr(bear_call)} not @beartype call.')
+    assert isinstance(func_wrapper_code, str), (
+        f'{repr(func_wrapper_code)} not string.')
+    assert isinstance(pith_repr, str), f'{repr(pith_repr)} not string.'
+    assert isinstance(hint_forwardrefs_class_basename, Iterable), (
+        f'{repr(hint_forwardrefs_class_basename)} not iterable.')
+
+    # Generate an unmemoized parameter-specific code snippet type-checking this
+    # parameter by replacing in this parameter-agnostic code snippet...
+    func_wrapper_code = replace_str_substrs(
+        text=func_wrapper_code,
+        # This placeholder substring cached into this code with...
+        old=PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
+        # This object representation of the name of this parameter or return.
+        new=pith_repr,
+    )
+
+    # If this code contains one or more relative forward reference placeholder
+    # substrings memoized into this code, unmemoize this code by globally
+    # resolving these placeholders relative to the decorated callable.
+    if hint_forwardrefs_class_basename:
+        # Callable currently being decorated by @beartype.
+        func = bear_call.func_wrappee
+
+        # Pass the beartypistry singleton as a private "__beartypistry"
+        # parameter to this wrapper function.
+        bear_call.func_wrapper_locals[ARG_NAME_TYPISTRY] = bear_typistry
+
+        # For each unqualified classname referred to by a relative forward
+        # reference type hints visitable from the current root type hint...
+        for hint_forwardref_class_basename in hint_forwardrefs_class_basename:
+            # Generate an unmemoized callable-specific code snippet
+            # type-checking this class by globally replacing in this
+            # callable-agnostic code...
+            func_wrapper_code = replace_str_substrs(
+                text=func_wrapper_code,
+                # This placeholder substring cached into this code with...
+                old=(
+                    f'{PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_PREFIX}'
+                    f'{hint_forwardref_class_basename}'
+                    f'{PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_SUFFIX}'
+                ),
+                # Python expression evaluating to this class when accessed
+                # via the private "__beartypistry" parameter.
+                new=register_typistry_forwardref(
+                    # Fully-qualified classname referred to by this forward
+                    # reference relative to the decorated callable.
+                    get_hint_pep484585_forwardref_classname_relative_to_object(
+                        hint=hint_forwardref_class_basename, obj=func,)),
+            )
+
+    # Return this unmemoized callable-specific code snippet.
+    return func_wrapper_code
