@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -22,7 +22,7 @@ submodule implementing a fine-grained object cache unique to those objects. For
 example:
 
 * This submodule unconditionally pools all instances of the
-  :class:`beartype._decor._data.BeartypeData` class under that type.
+  :class:`beartype._decor._call.BeartypeCall` class under that type.
 * The parallel :mod:`beartype._util.cache.pool.utilcachepoollistfixed`
   submodule conditionally pools every instance of the
   :class:`beartype._util.cache.pool.utilcachepoollistfixed.FixedList` class of
