@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -14,7 +14,7 @@ This private submodule is *not* intended for importation by downstream callers.
 # ....................{ IMPORTS                           }....................
 import sys
 from beartype.roar._roarexc import _BeartypeUtilCallableException
-from beartype._util.utiltyping import TypeException
+from beartype._data.datatyping import TypeException
 from types import FrameType
 from typing import Callable, Iterable, Optional
 
