@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -14,8 +14,7 @@ from beartype.roar import BeartypeDecorHintPep593Exception
 from beartype._data.hint.pep.sign.datapepsigncls import HintSign
 from beartype._data.hint.pep.sign.datapepsigns import HintSignAnnotated
 from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_7
-from beartype._util.utiltyping import TypeException
-from beartype.vale._valevale import BeartypeValidator
+from beartype._data.datatyping import TypeException
 from typing import Any, Optional, Tuple
 
 # See the "beartype.cave" submodule for further commentary.
@@ -40,7 +39,7 @@ def die_unless_hint_pep593(
     Parameters
     ----------
     hint : object
-        Object to be inspected.
+        Type hint to be inspected.
     exception_cls : TypeException
         Type of exception to be raised. Defaults to
         :exc:`BeartypeDecorHintPep593Exception`.
@@ -54,20 +53,42 @@ def die_unless_hint_pep593(
         If this object is *not* a :pep:`593`-compliant type metahint.
     '''
 
-    # Avoid circular import dependencies.
-    from beartype._util.hint.pep.utilpepget import (
-        get_hint_pep_sign_or_none)
-
     # If this hint is *NOT* PEP 593-compliant, raise an exception.
-    if get_hint_pep_sign_or_none(hint) is not HintSignAnnotated:
+    if not is_hint_pep593(hint):
         assert isinstance(exception_prefix, str), (
             f'{repr(exception_prefix)} not string.')
         raise exception_cls(
-            f'{exception_prefix}PEP 593 type hint {repr(hint)} '
-            f'neither "typing.Annotated" nor "typing_extensions.Annotated".'
+            f'{exception_prefix}type hint {repr(hint)} not PEP 593-compliant '
+            f'(e.g., "typing.Annotated[...]", '
+            f'"typing_extensions.Annotated[...]").'
         )
 
 # ....................{ TESTERS                           }....................
+#FIXME: Unit test us up.
+def is_hint_pep593(hint: Any) -> bool:
+    '''
+    ``True`` only if the passed object is a :pep:`593`-compliant **type
+    metahint** (i.e., subscription of either the :attr:`typing.Annotated` or
+    :attr:`typing_extensions.Annotated` type hint factories).
+
+    Parameters
+    ----------
+    hint : Any
+        Type hint to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a :pep:`593`-compliant type metahint.
+    '''
+
+    # Avoid circular import dependencies.
+    from beartype._util.hint.pep.utilpepget import get_hint_pep_sign_or_none
+
+    # Return true only if this hint is PEP 593-compliant.
+    return get_hint_pep_sign_or_none(hint) is HintSignAnnotated
+
+
 def is_hint_pep593_ignorable_or_none(
     hint: object, hint_sign: HintSign) -> Optional[bool]:
     '''
@@ -130,8 +151,7 @@ def is_hint_pep593_beartype(hint: Any) -> bool:
     Parameters
     ----------
     hint : Any
-        :pep:`593`-compliant :attr:`typing.Annotated` type hint to be
-        inspected.
+        :pep:`593`-compliant type hint to be inspected.
 
     Returns
     ----------
@@ -145,6 +165,9 @@ def is_hint_pep593_beartype(hint: Any) -> bool:
         If this object is *not* a :pep:`593`-compliant type metahint.
     '''
 
+    # Defer heavyweight imports.
+    from beartype.vale._core._valecore import BeartypeValidator
+
     # If this object is *NOT* a PEP 593-compliant type metahint, raise an
     # exception.
     die_unless_hint_pep593(hint)
@@ -232,8 +255,7 @@ else:
         return hint.__args__[0]
 
 
-get_hint_pep593_metadata.__doc__ = (
-    '''
+get_hint_pep593_metadata.__doc__ = '''
     Tuple of one or more arbitrary objects annotating the passed
     :pep:`593`-compliant **type metahint** (i.e., subscription of the
     :attr:`typing.Annotated` singleton).
@@ -265,11 +287,10 @@ get_hint_pep593_metadata.__doc__ = (
     ----------
     :func:`get_hint_pep593_metahint`
         Related getter.
-    ''')
-
-
-get_hint_pep593_metahint.__doc__ = (
     '''
+
+
+get_hint_pep593_metahint.__doc__ = '''
     PEP-compliant type hint annotated by the passed :pep:`593`-compliant **type
     metahint** (i.e., subscription of the :attr:`typing.Annotated` singleton).
 
@@ -301,4 +322,4 @@ get_hint_pep593_metahint.__doc__ = (
     ----------
     :func:`get_hint_pep593_metadata`
         Related getter.
-    ''')
+    '''
