@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 """
@@ -15,14 +15,30 @@ from beartype.roar._roarwarn import _BeartypeUtilCallableWarning
 from beartype._cave._cavefast import NumberType
 from beartype._util.utilobject import get_object_basename_scoped
 from collections.abc import Callable
-from pprint import saferepr
-from re import (
-    DOTALL,
-    sub as re_sub
-)
 from string import punctuation
+from typing import Dict
 
-# ....................{ PRIVATE                           }....................
+# ....................{ PRIVATE ~ cache                   }....................
+_MAX_LEN_TO_MAX_CHARS_REGEX: Dict[int, str] = {}
+'''
+Non-thread-safe global dictionary cache mapping from each **maximum length**
+(i.e., ``max_len`` parameter previously passed to the :func:`represent_object`
+function) to a compiled regular expression grouping zero or more leading
+characters preceding this maximum length *and* zero or more trailing
+delimiters.
+
+Caveats
+----------
+**This cache is intentionally non-thread-safe.** Although rendering this cache
+thread-safe would be trivial (e.g., via the
+:class:`beartype._util.cache.map.utilmapbig.CacheUnboundedStrong` class), doing
+so would both reduce efficiency *and* be ineffectual. Since this cache is
+*only* used to amortize the costs of regular expression compilation, violating
+thread-safety has *no* harmful side effects (aside from recompiling a
+previously compiled regular expression in unlikely edge cases).
+'''
+
+# ....................{ PRIVATE ~ frozen                  }....................
 _CHARS_PUNCTUATION = frozenset(punctuation)
 '''
 Frozen set of all **ASCII punctuation characters** (i.e., non-Unicode
@@ -49,7 +65,6 @@ to these representations either being effectively quoted already *or*
 sufficiently terse as to not benefit from being quoted).
 '''
 
-
 # ....................{ REPRESENTERS                      }....................
 def represent_object(
     # Mandatory parameters.
@@ -79,7 +94,7 @@ def represent_object(
            >>> def wat(mate: typing.List[str]) -> int: return len(mate)
            >>> raise_pep_call_exception(
            ...     func=muh_func, pith_name='mate', pith_value=[7,])
-           beartype.roar.BeartypeCallHintPepParamException: @beartyped wat()
+           beartype.roar.BeartypeCallHintParamViolation: @beartyped wat()
            parameter mate=[7] violates PEP type hint typing.List[str], as list
            item 0 value 7 not a str.
 
@@ -118,15 +133,41 @@ def represent_object(
     """
     assert isinstance(max_len, int), f'{repr(max_len)} not integer.'
 
-    # String describing the passed object. Note that:
+    #FIXME: Render this safe against infinitely recursive data structures.
+    #Unfortunately, we *CANNOT* call the standard pprint.saferepr() function to
+    #do so, as that function is *OUTRAGEOUSLY* slow on worst-case edge cases.
+    #Instead, we'll need to implement our own performant saferepr()
+    #alternative. Fortunately, note that someone's already done so: the popular
+    #BSD-licensed Celerity project, whose celerity.utils.saferepr.saferepr()
+    #function claims to actually be faster than the repr() builtin under
+    #certain circumstances. While impressive, repurposing Celerity's saferepr()
+    #implementation for @beartype will be non-trivial; that function internally
+    #leverages a number of non-trivial internal functions, including a
+    #streaming iterator that appears to be performing some sort of ad-hoc
+    #tokenization (!) on the input object's string representation. Although
+    #that submodule is less than 300 lines, that's 300 *INTENSE* lines.
+    #Nonetheless, we'll need to do this sooner or later. Currently, later. By
+    #the time you read this next, probably sooner. Until someone pounds their
+    #fists on our issue tracker, let's pretend this isn't a compelling concern.
+    #See also:
+    #   https://github.com/celery/celery/blob/master/celery/utils/saferepr.py
+
+    # String describing this object. Note that:
     # * This representation quote-protects all newlines in this representation.
     #   Ergo, "\n" *MUST* be matched as r"\n" instead below.
     # * For debuggability, the verbose (albeit less readable) output of repr()
     #   is preferred to the terse (albeit more readable) output of str().
-    # * For safety, the saferepr() function explicitly protected against
-    #   recursive data structures is preferred to the unsafe repr() builtin
-    #   *NOT* protected against such recursion.
-    obj_repr = saferepr(obj)
+    # * For safety, the pprint.saferepr() function explicitly protected against
+    #   recursive data structures *WOULD* typically be preferred to the unsafe
+    #   repr() builtin *NOT* protected against such recursion. Sadly,
+    #   pprint.saferepr() is extremely unoptimized and thus susceptible to
+    #   extreme performance regressions when passed a worst-case object (e.g.,
+    #   deeply nested container).
+    obj_repr = repr(obj)
+
+    #FIXME: Uncomment to exhibit a performance regression.
+    # from pprint import saferepr
+    # obj_repr = saferepr(obj)
 
     # If this representation is empty, return empty double-quotes. Although
     # most objects (including outlier singletons like "None" and the empty
@@ -156,43 +197,19 @@ def represent_object(
         # If this maximum length is long enough to at least allow truncation to
         # ellipses (i.e., a substring of length 3)...
         if max_len > 3:
-            #FIXME: Consider compiling and caching this regex into a module
-            #dictionary mapping from "max_len" values to compiled regexes.
+            # Last character of this representation.
+            obj_repr_char_last = obj_repr[max_len-1]
 
-            # Uncompiled regular expression grouping zero or more leading
-            # characters preceding this maximum length *AND* zero or more
-            # trailing delimiters.
-            #
-            # Note that:
-            # * This expression conditionally depends on a passed parameter and
-            #   thus *CANNOT* be compiled as a module-scoped global.
-            # * Efficiency is *NOT* a consideration for this function, which is
-            #   only called in the edge-case event of a runtime error.
-            PRE_MAX_CHARS_LAST_DELIMITERS_REGEX = (
-                # Group anchored at the string start preserving a maximum
-                # number (excluding the length required to inject an ellipses)
-                # minus one to account for the fact that both bounds of the
-                # "{...,...}" range operator are inclusive) of any characters.
-                r'^(.{0,'
-                f'{max_len - 4}'
-                r'})'
-                # Ungrouped remainder to be truncated.
-                r'.*?'
-                # Group anchored at the string end preserving zero or more
-                # trailing delimiters.
-                r'([\'")}>\]]*)$'
-            )
-
-            # Replace the substring of this representation from whichever of the
-            # first character following this maximum length or the first newline
-            # occurs first to the string end (excluding any optional trailing
-            # delimiters) with a single ellipses.
-            obj_repr = re_sub(
-                PRE_MAX_CHARS_LAST_DELIMITERS_REGEX,
-                r'\1...\2',
-                obj_repr,
-                flags=DOTALL,
-            )
+            # If this character is punctuation, preserve this punctuation by
+            # replacing the trailing portion of this representation up to but
+            # *NOT* including this punctuation with an ellipses.
+            if obj_repr_char_last in _CHARS_PUNCTUATION:
+                obj_repr = f'{obj_repr[:max_len-4]}...{obj_repr_char_last}'
+            # Else, this character is *NOT* punctuation. In this case, replace
+            # the trailing portion of this representation (including this
+            # character) with an ellipses.
+            else:
+                obj_repr = f'{obj_repr[:max_len-3]}...'
         # Else, this maximum length is *NOT* long enough to at least allow
         # truncation to ellipses (i.e., a substring of length 3). In this case,
         # truncate this string to this length *WITHOUT* ellipses.
@@ -243,16 +260,17 @@ def represent_func(
     from beartype._util.func.utilfunccode import get_func_code_or_none
     from beartype._util.func.utilfunctest import is_func_lambda
 
-    # Return either...
-    return (
-        # If this callable is a pure-Python lambda function, either:
-        # * If this lambda is defined by an on-disk script or module source
-        #   file, the exact substring of that file defining this lambda.
-        # * Else (e.g., if this lambda is dynamically defined in-memory), a
-        #   placeholder string.
-        get_func_code_or_none(func=func, warning_cls=warning_cls) or '<lambda>'
-        if is_func_lambda(func) else
-        # Else, this callable is *NOT* a pure-Python lambda function. In this
-        # case, the fully-qualified name of this non-lambda function.
-        f'{get_object_basename_scoped(func)}()'
-    )
+    # If this callable is a pure-Python lambda function, return either:
+    # * If this lambda is defined by an on-disk script or module source file,
+    #   the exact substring of that file defining this lambda.
+    # * Else (e.g., if this lambda is dynamically defined in-memory), a
+    #   placeholder string.
+    if is_func_lambda(func):
+        return (
+            get_func_code_or_none(func=func, warning_cls=warning_cls) or
+            '<lambda>'
+        )
+    # Else, this callable is *NOT* a pure-Python lambda function.
+
+    # Return the fully-qualified name of this non-lambda function.
+    return get_object_basename_scoped(func)
