@@ -11,11 +11,6 @@ This private submodule implements a :func:`beartype.beartype``-compatible
 :class:`typing.Protocol` that can lead to significant performance improvements.
 '''
 
-# ....................{ TODO                              }....................
-#FIXME: *UHOH.* Turns out caching protocols fail to support a standard use case
-#of subscription by non-type variables satisfying a type variable. See also:
-#    https://github.com/beartype/beartype/pull/86#issuecomment-1042681735
-
 # ....................{ IMPORTS                           }....................
 from beartype._util.py.utilpyversion import (
     IS_PYTHON_AT_LEAST_3_8,
@@ -27,21 +22,22 @@ from beartype._util.py.utilpyversion import (
 # 544...
 #
 # This is one of those cases where one pines for a module-scope return
-# statement. (I seem to remember a bug/feature request about that somewhere,
-# but couldn't find it after a brief search.)
+# statement. I seem to remember a bug/feature request about that somewhere,
+# but couldn't find it after a brief search.
 if IS_PYTHON_AT_LEAST_3_8:
     # ..................{ IMPORTS                           }..................
+    #FIXME: The ignore[attr-defined] is for Python 3.7 because Mypy doesn't
+    #understand IS_PYTHON_AT_LEAST_3_8. That ignore should be removable
+    #when retiring PYTHON 3.7.
+    # Defer Python version-specific imports, including non-caching
+    # protocols to be overridden by caching equivalents below (and other
+    # requirements from various sources, depending on runtime environment).
     from beartype._util.cache.utilcachecall import callable_cached
     from typing import (  # type: ignore[attr-defined]
         EXCLUDED_ATTRIBUTES,
         TYPE_CHECKING,
         Any,
         Generic,
-        TypeVar,
-        Union,
-        runtime_checkable,
-
-        # Non-caching protocols to be overridden by caching equivalents below.
         Protocol as _ProtocolSlow,
         SupportsAbs as _SupportsAbsSlow,
         SupportsBytes as _SupportsBytesSlow,
@@ -50,11 +46,14 @@ if IS_PYTHON_AT_LEAST_3_8:
         SupportsIndex as _SupportsIndexSlow,
         SupportsInt as _SupportsIntSlow,
         SupportsRound as _SupportsRoundSlow,
+        TypeVar,
+        Union,
+        runtime_checkable,
     )
 
     # Note that this branch is intentionally tested first, despite the
     # resulting negation. Why? Because mypy quietly defecates all over itself
-    # if the order of these two branches is reversed.
+    # if the order of these two branches is reversed. It's as bad as it sounds.
     if not IS_PYTHON_AT_LEAST_3_9:
         from typing import Dict, Iterable, Tuple, Type  # type: ignore[misc]
     # Else, the active Python interpreter targets Python >= 3.9 and thus
@@ -189,7 +188,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         ) -> _TT:
 
             # See <https://github.com/python/mypy/issues/9282>
-            cls = super().__new__(mcls, name, bases, namespace, **kw)  # type: ignore [misc]
+            cls = super().__new__(mcls, name, bases, namespace, **kw)  # type: ignore[misc]
 
             # Mark this protocol class as a runtime protocol. By default,
             # "typing.Protocol" subclasses are only static-time. Although
@@ -343,7 +342,16 @@ if IS_PYTHON_AT_LEAST_3_8:
 
     # ..................{ CLASSES                           }..................
     @runtime_checkable
-    class Protocol(_ProtocolSlow, metaclass=_CachingProtocolMeta):
+    class Protocol(
+        _ProtocolSlow,
+        # Force protocols to be generics. Although the standard
+        # "typing.Protocol" superclass already implicitly subclasses from the
+        # "typing.Generic" superclass, the non-standard
+        # "typing_extensions.Protocol" superclass does *NOT*. Ergo, we force
+        # this to be the case.
+        Generic,
+        metaclass=_CachingProtocolMeta,
+    ):
         '''
         :func:`beartype.beartype`-compatible (i.e., decorated by
         :func:`typing.runtime_checkable`) drop-in replacement for
@@ -435,8 +443,8 @@ if IS_PYTHON_AT_LEAST_3_8:
                     # variables *OR* types satisfying the type variables
                     # subscripting the superclass (e.g.,
                     # "UserDefinedProtocol[str]" for a user-defined subclass
-                    # class UserDefinedProtocol(Protocol[AnyStr]), that
-                    # subclass as is.
+                    # class UserDefinedProtocol(Protocol[AnyStr]). In this
+                    # case, this subclass as is.
                     cls
                 )
 
@@ -486,6 +494,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsAbs`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -493,6 +502,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsBytes`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -500,6 +510,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsComplex`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -507,6 +518,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsFloat`."
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -514,6 +526,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsInt`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -521,6 +534,7 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsIndex`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
 
 
@@ -528,4 +542,5 @@ if IS_PYTHON_AT_LEAST_3_8:
         '''
         Caching variant of :class:`typing.SupportsRound`.
         '''
+        __module__: str = 'beartype.typing'
         __slots__: Union[str, Iterable[str]] = ()
