@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -17,12 +17,25 @@ from beartype._cave._cavefast import (
     NoneType,
 )
 from beartype._cave._cavemap import NoneTypeOr
+from beartype_test.a00_unit.data.hint.nonpep.data_nonpep import (
+    HINTS_NONPEP_META,
+)
 from beartype_test.a00_unit.data.hint.pep.data_pep import (
     HINTS_PEP_HASHABLE,
     HINTS_PEP_IGNORABLE_SHALLOW,
     HINTS_PEP_IGNORABLE_DEEP,
+    HINTS_PEP_META,
 )
 
+# ....................{ HINTS ~ tuples                    }....................
+HINTS_META = HINTS_PEP_META + HINTS_NONPEP_META
+'''
+Tuple of all **PEP-agnostic type hint metadata** (i.e.,
+:class:`HintPepMetadata` instances describing test-specific type hints with
+metadata leveraged by various testing scenarios -- including both PEP-compliant
+and -noncompliant type hints).
+'''
+
 # ....................{ NON-HINTS ~ sets                  }....................
 NOT_HINTS_HASHABLE = frozenset((
     # Scalar that is neither a type nor string (i.e., forward reference).
