@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -94,10 +94,12 @@ def _init() -> None:
         _data_pep586,
         _data_pep589,
         _data_pep593,
+        _data_pep604,
     )
 
     # Submodule globals to be redefined below.
     global \
+        HINTS_META, \
         HINTS_PEP_HASHABLE, \
         HINTS_PEP_IGNORABLE_DEEP, \
         HINTS_PEP_IGNORABLE_SHALLOW, \
@@ -116,6 +118,7 @@ def _init() -> None:
         _data_pep586,
         _data_pep589,
         _data_pep593,
+        _data_pep604,
     )
 
     # Initialize all private submodules of this subpackage.
